@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Jeffrey Stedfast <fejj@helixcode.com>
- *
- *  Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -194,8 +191,10 @@ verify_foreach_callback (GMimeObject *part, gpointer user_data)
 	if (GMIME_IS_MULTIPART_SIGNED (part)) {
 		/* this is a multipart/signed part, so we can verify the pgp signature */
 		GMimeMultipartSigned *mps = (GMimeMultipartSigned *) part;
-		GMimeCipherValidity *validity;
+		GMimeSignatureValidity *validity;
+		GMimeSignatureStatus status;
 		GError *err = NULL;
+		const char *str;
 		
 		if (!(validity = g_mime_multipart_signed_verify (mps, ctx, &err))) {
 			/* an error occured - probably couldn't start gpg? */
@@ -208,10 +207,26 @@ verify_foreach_callback (GMimeObject *part, gpointer user_data)
 			g_error_free (err);
 		} else {
 			/* print out validity info - GOOD vs BAD and "why" */
-			printf ("PGP signature is %s:\n%s\n", g_mime_cipher_validity_get_valid (validity) ? "GOOD" : "BAD",
-				g_mime_cipher_validity_get_description (validity));
+			status = g_mime_signature_validity_get_status (validity);
+			switch (status) {
+			case GMIME_SIGNATURE_STATUS_GOOD:
+				str = "Good";
+				break;
+			case GMIME_SIGNATURE_STATUS_BAD:
+				str = "Bad";
+				break;
+			case GMIME_SIGNATURE_STATUS_UNKNOWN:
+				str = "Unknown";
+				break;
+			default:
+				str = NULL;
+				break;
+			}
 			
-			g_mime_cipher_validity_free (validity);
+			printf ("PGP signature is %s:\n%s\n", str,
+				g_mime_signature_validity_get_details (validity));
+			
+			g_mime_signature_validity_free (validity);
 		}
 	}
 }
