@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -65,6 +65,7 @@
  * @short_description: Charset helper functions
  * @see_also:
  *
+ * Charset utility functions.
  **/
 
 
@@ -904,27 +905,15 @@ g_mime_charset_can_encode (GMimeCharset *mask, const char *charset, const char *
 void
 g_mime_set_user_charsets (const char **charsets)
 {
-	int i, n = 0;
-	
 	if (user_charsets)
 		g_strfreev (user_charsets);
 	
-	if (charsets == NULL) {
+	if (charsets == NULL || charsets[0] == NULL) {
 		user_charsets = NULL;
 		return;
 	}
 	
-	while (charsets[n] != NULL)
-		n++;
-	
-	if (n == 0) {
-		user_charsets = NULL;
-		return;
-	}
-	
-	user_charsets = g_malloc (sizeof (char *) * (n + 1));
-	for (i = 0; i <= n; i++)
-		user_charsets[i] = (char *) charsets[i];
+	user_charsets = g_strdupv ((char **) charsets);
 }
 
 
