@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -38,6 +39,18 @@
 #define d(x)
 
 
+/**
+ * SECTION: gmime-multipart-encrypted
+ * @title: GMimeMultpartEncrypted
+ * @short_description: Encrypted MIME multiparts
+ * @see_also: #GMimeMultipart
+ *
+ * A #GMimeMultipartEncrypted part is a special subclass of
+ * #GMimeMultipart to make it easier to manipulate the
+ * multipart/encrypted MIME type.
+ **/
+
+
 /* GObject class methods */
 static void g_mime_multipart_encrypted_class_init (GMimeMultipartEncryptedClass *klass);
 static void g_mime_multipart_encrypted_init (GMimeMultipartEncrypted *mps, GMimeMultipartEncryptedClass *klass);
@@ -224,11 +237,10 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 				    GMimeCipherContext *ctx, GPtrArray *recipients,
 				    GError **err)
 {
+	GMimeStream *filtered_stream, *ciphertext, *stream;
 	GMimePart *version_part, *encrypted_part;
 	GMimeContentType *content_type;
 	GMimeDataWrapper *wrapper;
-	GMimeStream *filtered_stream;
-	GMimeStream *stream, *ciphertext;
 	GMimeFilter *crlf_filter;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), -1);
@@ -265,10 +277,14 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	
 	/* construct the version part */
 	version_part = g_mime_part_new ();
-	g_mime_part_set_encoding (version_part, GMIME_PART_ENCODING_7BIT);
-	g_mime_part_set_content (version_part, "Version: 1\n", strlen ("Version: 1\n"));
 	content_type = g_mime_content_type_new_from_string (ctx->encrypt_protocol);
 	g_mime_object_set_content_type (GMIME_OBJECT (version_part), content_type);
+	g_mime_part_set_encoding (version_part, GMIME_PART_ENCODING_7BIT);
+	stream = g_mime_stream_mem_new_with_buffer ("Version: 1\n", strlen ("Version: 1\n"));
+	wrapper = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_7BIT);
+	g_mime_part_set_content_object (version_part, wrapper);
+	g_object_unref (wrapper);
+	g_object_unref (stream);
 	
 	mpe->protocol = g_strdup (ctx->encrypt_protocol);
 	mpe->decrypted = content;
@@ -276,19 +292,18 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	
 	/* construct the encrypted mime part */
 	encrypted_part = g_mime_part_new_with_type ("application", "octet-stream");
-	wrapper = g_mime_data_wrapper_new ();
-	g_mime_data_wrapper_set_stream (wrapper, ciphertext);
-	g_object_unref (ciphertext);
-	g_mime_part_set_content_object (encrypted_part, wrapper);
 	g_mime_part_set_encoding (encrypted_part, GMIME_PART_ENCODING_7BIT);
+	wrapper = g_mime_data_wrapper_new_with_stream (ciphertext, GMIME_PART_ENCODING_7BIT);
+	g_mime_part_set_content_object (encrypted_part, wrapper);
+	g_object_unref (ciphertext);
 	g_object_unref (wrapper);
 	
 	/* save the version and encrypted parts */
 	/* FIXME: make sure there aren't any other parts?? */
 	g_mime_multipart_add_part (GMIME_MULTIPART (mpe), GMIME_OBJECT (version_part));
-	g_object_unref (version_part);
 	g_mime_multipart_add_part (GMIME_MULTIPART (mpe), GMIME_OBJECT (encrypted_part));
 	g_object_unref (encrypted_part);
+	g_object_unref (version_part);
 	
 	/* set the content-type params for this multipart/encrypted part */
 	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mpe), "protocol", mpe->protocol);
